'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var _require = require('./util'),
    subscribeEventObservables = _require.subscribeEventObservables,
    createLifecycleSubjects = _require.createLifecycleSubjects;

var makeInteractions = require('./interactions');

function createReactClass(React, Adapter, createCycleComponent, createRenderer, isTemplateComponent) {
  var makePropsObservable = Adapter.makePropsObservable,
      createEventSubject = Adapter.createEventSubject,
      CompositeDisposable = Adapter.CompositeDisposable,
      next = Adapter.next,
      complete = Adapter.complete,
      isObservable = Adapter.isObservable;


  return function component(displayName, definitionFn, templateFn, componentOptions) {
    if (typeof displayName !== 'string') {
      throw new Error('Invalid displayName');
    }
    if (typeof definitionFn !== 'function') {
      throw new Error('Invalid definitionFn');
    }
    if (isTemplateComponent && typeof templateFn !== 'function') {
      throw new Error('Invalid templateFn');
    }

    var options = (
    // option isTemplateComponent (y): 4th argument (n): 3rd argument
    isTemplateComponent ? componentOptions : templateFn) || {};

    var ReactClass = function (_React$Component) {
      _inherits(ReactClass, _React$Component);

      function ReactClass(props) {
        _classCallCheck(this, ReactClass);

        var _this = _possibleConstructorReturn(this, (ReactClass.__proto__ || Object.getPrototypeOf(ReactClass)).call(this, props));

        _this.hasNewValue = false;
        _this.state = {
          newValue: null
        };
        return _this;
      }

      _createClass(ReactClass, [{
        key: '_subscribeCycleComponent',
        value: function _subscribeCycleComponent() {
          var _this2 = this;

          this.disposable = new CompositeDisposable();
          this.propsSubject$ = makePropsObservable(this.props);
          this.interactions = makeInteractions(createEventSubject);
          this.lifecycles = isTemplateComponent ? createLifecycleSubjects(createEventSubject) : null;

          this.cycleComponent = createCycleComponent(isObservable, definitionFn, this.interactions, this.propsSubject$, this.lifecycles);
          var subscription = this.cycleComponent.newValue$.subscribe(function (newValue) {
            _this2.hasNewValue = true;
            _this2.setState({ newValue: newValue });
          });

          this.disposable.add(this.propsSubject$);
          this.disposable.add(subscription);

          var cycleComponentDispose = this.cycleComponent.unsubscribe;
          if (cycleComponentDispose) {
            if (typeof cycleComponentDispose === 'function' || typeof cycleComponentDispose.unsubscribe === 'function') {
              this.disposable.add(cycleComponentDispose);
            }
          }
        }
      }, {
        key: '_unsubscribeCycleComponent',
        value: function _unsubscribeCycleComponent() {
          if (this.propsSubject$) {
            complete(this.propsSubject$);
          }
          if (this.disposable) {
            this.disposable.unsubscribe();
          }
        }
      }, {
        key: '_subscribeCycleEvents',
        value: function _subscribeCycleEvents() {
          var subscriptions = subscribeEventObservables(this.cycleComponent.customEvents, this);
          if (subscriptions.length > 0) {
            for (var i = 0; i < subscriptions.length; i++) {
              this.disposable.add(subscriptions[i]);
            }
          }
        }
      }, {
        key: 'shouldComponentUpdate',
        value: function shouldComponentUpdate(nextProps, nextState) {
          // Only care about the state since the props have been observed.
          return this.state !== nextState;
        }
      }, {
        key: 'componentWillMount',
        value: function componentWillMount() {
          // componentWillMount is called for both client and server
          // https://facebook.github.io/react/docs/component-specs.html#mounting-componentwillmount
          this._subscribeCycleComponent();
          if (isTemplateComponent) {
            next(this.lifecycles.componentWillMount);
          }
        }
      }, {
        key: 'componentDidMount',
        value: function componentDidMount() {
          this._subscribeCycleEvents();
          if (isTemplateComponent) {
            next(this.lifecycles.componentDidMount);
          }
        }
      }, {
        key: 'componentWillReceiveProps',
        value: function componentWillReceiveProps(nextProps) {
          next(this.propsSubject$, nextProps);
          if (isTemplateComponent) {
            next(this.lifecycles.componentWillReceiveProps, nextProps);
          }
        }
      }, {
        key: 'componentWillUpdate',
        value: function componentWillUpdate(nextProps) {
          if (isTemplateComponent) {
            next(this.lifecycles.componentWillUpdate, nextProps);
          }
        }
      }, {
        key: 'componentDidUpdate',
        value: function componentDidUpdate(prevProps) {
          if (isTemplateComponent) {
            next(this.lifecycles.componentDidUpdate, prevProps);
          }
        }
      }, {
        key: 'componentWillUnmount',
        value: function componentWillUnmount() {
          // componentWillUnmount is not being called for the server context
          if (isTemplateComponent) {
            complete(this.lifecycles.componentWillMount);
            complete(this.lifecycles.componentDidMount);
            complete(this.lifecycles.componentWillReceiveProps);
            complete(this.lifecycles.componentWillUpdate);
            complete(this.lifecycles.componentDidUpdate);
            next(this.lifecycles.componentWillUnmount);
            complete(this.lifecycles.componentWillUnmount);
          }
          this._unsubscribeCycleComponent();
        }
      }]);

      return ReactClass;
    }(React.Component);

    ReactClass.prototype.render = createRenderer(React, templateFn);

    ReactClass.displayName = displayName;
    if (options.propTypes) {
      ReactClass.propTypes = options.propTypes;
    }

    return ReactClass;
  };
}

module.exports = createReactClass;